direct page addresses

;
;  This work was originally done by Fred Taft (fred@hp-pcd.cv.hp.com).
;  Please forward any comments, corrections or additions back to Fred.
;
;  Mine Storm
;

      .ORG   $E000

MS_header:
       DW    #MS_music                    ; Mine Storm music block pointer
       DB    $F8                          ; height = -8
       DB    $50                          ; width = 80
       DB    $30                          ; rel y = 48
       DB    $E8                          ; rel x = -24
       DB    "MINE",$80
       DB    $F8                          ; height = -8
       DB    $50                          ; width = 80
       DB    $00                          ; rel y = 0
       DB    $DE                          ; rel x = -34
       DB    "STORM",$80,$00

; Start of the Mine Storm game
MS_start:
       LDX   #$C883                       ; Clear C883 - CBC5
PE019: CLR   ,X+
       CMPX  #$CBC5
       BNE   PE019

       JSR   init_motion_dots
       INC   $C824
       LDA   #$BB
       STA   $C880                        ; Store console enable flags in C880
       LDX   #$0101
       STX   $C881                        ; Store jstick mak in C881

restart_game:
       LDX   #$C883                       ; Clear C883 - CB70
PE034: CLR   ,X+
       CMPX  #$CB70
       BNE   PE034
       BRA   PE03D

PE03D: JSR   dptoC8
       direct $c8
       LDD   #$0200                       ; Ask for number of players
       JSR   get_players_game
       DEC   $c879                        ; C879 contains # of players selected
       CLR   $c856
       CLR   $c89B                        ; C89B specifies which player is active

       LDX   #$C8A8
       JSR   set_dft_score                ; Initialize player 1 score
       LDX   #$C8AF
       JSR   set_dft_score                ; Initialize player 2 score
       LDX   #$C8F9
       JSR   set_dft_score                ; Initialize mine field # for player 1
       LDD   #$0001
       JSR   add_d_to_x_in_bcd
       LDX   #$C900
       JSR   set_dft_score                ; Initialize mine field # for player 2
       LDD   #$0001
       JSR   add_d_to_x_in_bcd

       LDX   #mines_at_level_x            ; Initialize player 1 & 2's pointers
       STX   $c8C4                        ; into the structure describing which
       STX   $c8C6                        ; mines are at which level.

       LDA   #$05
       STA   $c8D9                        ; Initialize # of ships for active player
       STA   $c8DA                        ; Initialize # of ships for player 1
       STA   $c8DB                        ; Initialize # of ships for player 2
       BRA   restart_same_level

start_next_mine_level:
       JSR   bring_ship_to_center
       LDY   #$C8C4                       ; Increment the pointer into the struct
       LDA   $c89B                        ; describing which mines are at which
       LDX   A,Y                          ; level, for the active player.
       LEAX  4,X
       STX   A,Y

       LDX   #mine_field_level_pointers
       LDA   $c89B                        ; Check the mine field level for
       LDX   A,X                          ; the active player, and if it is
       LDA   5,X                          ; a multiple of 4, then award that
       ANDA  #$03                         ; player a bonus ship.
       BNE   goto_next_mine_level
       INC   $c8D9

goto_next_mine_level:
       LDD   #$0001                       ; Increment active players mine
       JSR   add_d_to_x_in_bcd            ; field level indicator.
restart_same_level:
       JSR   init_RAM_for_new_level
       LDX   #$C8C4                       ; Use the pointer in C8C4 (player 1)
       LDA   $c89B                        ; or C8C6 (player 2) to determine
       LDX   A,X                          ; which mines are at this level.
       LDA   ,X
       BMI   check_4_end_of_game          ; After level 13, skip any level whose
       JSR   seed_the_screen              ; first byte is negative.
       BRA   main_loop

check_4_end_of_game:
       LDD   $c8F0                        ; If C8F0 counts down to 0, then
       SUBD  #$0001                       ; start a new game; used as a delay
       STD   $c8F0                        ; after a game is over.
       BEQ   start_a_new_game

       ; Display 'game over' string, & both scores
       PSHS  DP
       JSR   dptoD0
       direct $D0
       JSR   display_both_scores
       LDU   #game_over
       JSR   MS_print_1_string7F
       PULS  DP
       direct $c8

       LDA   $c80F                        ; If no buttons pressed, then see if
       BEQ   main_loop                    ; we should goto advanced levels.

       ; Find out if a new hi score is available,
       ;  and then start up a new game.
start_a_new_game:
       LDX   #$C8A8                       ; See if player 1 has new hi score
       LDU   #$CBEB
       JSR   check_4_new_hi_score
       LDX   #$C8AF                       ; See if player 2 has new hi score
       LDU   #$CBEB
       JSR   check_4_new_hi_score
       LDD   $c8F0
       LBNE  restart_game                 ; Just start a new game
       JSR   reinit
       CLR   $c83B                        ; Flag that a hi score should be displayed
       LDS   #$CBEA
       JMP   powerup_loop                 ; Go thru a complete reset sequence


 ; This is the main loop during a players turn.  Control will
 ; remain in this loop, until either a player clears a level,
 ; or has his ship destroyed.
main_loop:
       PSHS  DP
       JSR   process_indirect_jumps
       JSR   draw_non_dead_mines
       JSR   process_jstick_and_buttons
       JSR   process_bullets
       JSR   reseed_screen
       PULS  DP
       direct $c8
       JSR   check_bullets_for_hits
       JSR   check_ship_4_mine_collision
       JSR   check_ship_4_enemy_collision
       JSR   display_explosion_pattern
       BLO   main_loop
       LDA   $c8BD                        ; See if cmd ship is disabled
       LBEQ  start_next_mine_level
       LDA   $c8BE                        ; See if the game is over
       LBNE  check_4_end_of_game
       JMP   restart_same_level



 ; This block of code is responsible for seeding the screen
 ; at the beginning of play.  It will draw the enemy ship
 ; as it moves from the top of the screen to the bottom.
 ; After the field has been completely seeded, the mine
 ; field number will be displayed.
 ; I think that the x register, at entry, points to a
 ; structure describing the types of mines for this level.

seed_the_screen:
       STX   $c8C2                        ; Save pointer to mine info
       LDD   #$7F00
       STD   $c8DC                        ; C8DC contains enemy ship coords
       STA   $c8B7                        ; C8B7 is used as loop cntr ($7F)
       LDA   #$20
       STA   $c89C                        ; Set up 1 of the indirect
       LDX   #generate_mine_coords        ; jump ptrs, to generate some
       STX   $c89D                        ; mine coordinates.
       LDX   #$C933                       ; Load mine buf addr into C8B9-C8BA
       STX   $c8B9   
       LDA   #$1D                         ; Load # of mines into C8B8
       STA   $c8B8
       CLR   $c856
       LDU   #MS_music
       JSR   init_sound2
continue_seeding_screen:
       PSHS  DP
       JSR   PE711                        ; Draw active players ship count
       JSR   init_sound
       LDA   $c826                        ; Decrement our loop counter, C8B7
       BITA  #$01                         ; every other pass.
       BNE   PE15C
       DEC   $c8B7
PE15C: JSR   process_indirect_jumps

       ; This block draws the enemy ship
       JSR   display_both_scores
       JSR   do_sound
       JSR   draw_non_dead_mines
       JSR   intensity_to_5F
       LDB   $C8B7                        ; Wait until screen is seeded,
       BEQ   activate_4_mines             ; then activate 1st 4 mines.
       LDX   #enemy_ship_pt1
       LDY   $C8DC
       JSR   move_y_draw_x                ; Draw the enemy ship
       LDX   #enemy_ship_pt2
       JSR   move_y_draw_x
       LDX   #enemy_ship_pt3
       JSR   move_y_draw_x
       PULS  DP
       direct $c8
       DEC   $c8DC                        ; Decrement y coord for enemy ship
       BRA   continue_seeding_screen

 ;
 ; This routine is called at the beginning of a turn for the
 ; active player.  It will activate the first 4 mines for
 ; the level.  The type of mines activated are controlled
 ; by the array whose pointer is contained in C8C2.
 ;
activate_4_mines:
       PULS  DP
       direct $c8
       CLR   $c89C
       LDA   #$04                         ; Use C8B7 as a loop counter
       STA   $c8B7                        ; Use C8B8 as a delay counter, when it
       LDA   #$7F                         ; = 0, stop displaying the mine field
       STA   $c8B8                        ; level, and activate 1st 4 mine.
PE198: LDA   $c8B7  
       BEQ   PE1E6                        ; Exit when loop counter = 0
       LDB   $c8B8
       BEQ   PE1A4                        ; Don't activate the mines until the
       DEC   $c8B8                        ; counter in C8B8 = 0; this allows the
       BRA   PE1B6                        ; mine field level to stay displayed awhile
PE1A4: LDB   $c826
       ANDB  #$1F                         ; After every 31 times thru waitrecal(),
       BNE   PE1B6                        ; activate another mine.
       DECA
       STA   $c8B7                        ; Decrement loop counter
       LDX   $c8C2
       LDA   A,X                          ; Determine what type mine this should be
       LDB   #$03                         ; Set generation number to 'largest'.
       JSR   activate_a_mine
PE1B6: PSHS  dp
       JSR   process_indirect_jumps

       ; Tell user what mine field number this is
       JSR   intensity_to_7F
       LDU   #mine_field
       JSR   MS_print_1_string7F
       LDY   #$E0F8
       LDU   #mine_field_level_pointers
       LDA   $C89B
       LDU   A,U
       JSR   move_y_print_string

       JSR   draw_non_dead_mines
       JSR   process_jstick_and_buttons
       JSR   process_bullets
       PULS  DP
       direct $c8
       JSR   check_bullets_for_hits
       JSR   display_explosion_pattern
       BRA   PE198
PE1E6: RTS


 ;
 ; This block of code generates random coordinates
 ; for a single mine.  C8B9 contains a pointer to
 ; the appropriate entry in the mine structure.
 ; Before exiting, this routine will update the
 ; pointer in C8B9 to point to the next entry.
 ;
generate_mine_coords:
       DEC   $c8B8                        ; Decrement the mine counter (C8B8)
       BEQ   PE239                        ; and exit when it equals 0.
       INC   $c8ED                        ; Inc mine dot counter
       JSR   get_random_a
       ANDA  #$07                         ; Reset the ind jump counter, to
       ADDA  #$04                         ; generate the next set of mine
       STA   $c89C                        ; coordinates.
       LDU   $c8B9                        ; Load u register w/ mine structure ptr
       LDA   #$80                         ; Flag that a mine is in this slot
       STA   ,U                           ; Get the coordinates of enemy ship
       LDD   $c8DC
       ADDA  #$08
       STA   4,U                          ; Save y coordinate in 4th slot
       CLR   5,U
       STB   6,U                          ; Save x coordinate in 6th slot
       CLR   7,U
generate_final_x_for_mine:
       JSR   get_random_a                 ; Generate the final x position for
       TSTA                               ; this mine.
       BMI   PE21A
       CMPA  #$10
       BGE   PE214
       ADDA  #$0C
PE214: CMPA  #$60
       BLE   PE226
       BRA   generate_final_x_for_mine
PE21A: CMPA  #$F0
       BLE   PE220
       SUBA  #$0C
PE220: CMPA  #$A0
       BGE   PE226
       BRA   generate_final_x_for_mine

PE226: STA   $11,U
       TFR   A,B                          ; Determine what the delta movement
       SEX                                ; value will be for this mine: -1 or 1
       ORA   #$01
       STA   $10,U
       CLR   2,U                          ; Start with a scale factor of 0
       LEAY  $12,U                        ; Update C8B9 to point to next entry
       STY   $c8B9
PE239: RTS

final_mine_scale_factor:
       DB    $00
       DB    $02                          ; Small
       DB    $07                          ; Medium
       DB    $10                          ; Large

mine_velocity:
       DB    $00
       DB    $20                          ; Small
       DB    $18                          ; Medium
       DB    $10                          ; Large

mine_values:
       DB    $01                          ; dumb mine = BCD 100
       DB    $00
       DB    $05                          ; magnetic mine = BCD 500
       DB    $00
       DB    $03                          ; fireball mine = BCD 325
       DB    $25
       DB    $07                          ; magnetic fireball mine = BCD 750
       DB    $50

extra_mine_value:
       DB    $00
       DB    $00
       DB    $01                          ; Small (100)
       DB    $00
       DB    $00                          ; Medium (35)
       DB    $35
       DB    $00                          ; Large (0)
       DB    $00

mine_sizes:
       DB    $00
       DB    $00
       DB    $04                          ; Small
       DB    $04
       DB    $08                          ; Medium
       DB    $08
       DB    $0D                          ; Large
       DB    $0D

; Table of pointers to mine vectors
mine_vector_ptrs:
       DW    #dumb_mine
       DW    #magnetic_mine
       DW    #fireball_mine
       DW    #magnetic_fireball_mine


 ;
 ; This block of code processe the hyperspace and move
 ; buttons.  It also checks the state of the joystick,
 ; and updates the command ship rotation value, kept
 ; in C8D4, if the joystick is not centered.  If the
 ; joystick is left, then the rotation value is
 ; incremented; if it is to the right, then the rotation
 ; value is decremented.  After updating the rotation
 ; value, the command ship will be redrawn at its
 ; current position (specified in C8C8-C8C9).
 ;
process_jstick_and_buttons:
       PSHS  DP
       LDA   #$C8
       TFR   A,DP
       direct $c8
       LDA   $c8BD                        ; Skip if cmd ship disabled
       LBNE  PE30A
       LDA   $c8EE                        ; Continue hyperspace, if active
       LBNE  continue_hyperspace
       LDA   $c813                        ; Check the hyperspace button (2)
       LBNE  hyperspace
       LDA   $c814                        ; Check the move button (3)
       BEQ   no_ship_movement

       ;
       ; The following block of code takes care of
       ; generating the new command coordinates,
       ; because it is moving.  It attempts to make
       ; all turns into smooth motion.
       ;
       LDA   $c8D4
       CMPA  $c8D6
       BEQ   PE2A0
       CMPA  $c8D8
       BEQ   PE290
       LDA   $c8D5
       BEQ   PE2A0
       LDA   $c8D7
       BNE   no_ship_movement
PE290: LDA   $c8D7
       ADDA  #$0C
       CMPA  #$7F
       BHI   no_ship_movement
       STA   $c8D7
       LDA   $c8D4
       STA   $c8D8
       BRA   PE2AE
PE2A0: LDA   $c8D5
       ADDA  #$0C
       CMPA  #$7F
       BHI   no_ship_movement
       STA   $c8D5
       LDA   $c8D4
       STA   $c8D6
PE2AE: INC   $c8F2                        ; Flag that a sound should be made

no_ship_movement:
       LDA   $c8D5
       BEQ   PE2C2
       SUBA  #$02
       STA   $c8D5
       LDB   $c8D6
       JSR   calc_rise_run1
       STY   $c8CC
       STX   $c8CE
PE2C2: LDA   $c8D7
       BEQ   PE2D4
       SUBA  #$02
       STA   $c8D7
       LDB   $c8D8
       JSR   calc_rise_run1               ; Generate new rise & run
       STY   $c8D0
       STX   $c8D2
PE2D4: LDD   $c8C8                        ; Update cmd ships y coord
       ADDD  $c8CC
       ADDD  $c8D0
       STD   $c8C8
       LDD   $c8CA                        ; Update cmd ships x coord
       ADDD  $c8CE
       ADDD  $c8D2
       STD   $c8CA

       ; Check joystick state (in C81B)
       LDA   $c81B
       BEQ   draw_cmd_ship1               ; Joystick is centered
       BMI   PE2EE
       DEC   $c8D4                        ; Joystick to right
       BRA   draw_cmd_ship2
PE2EE: INC   $c8D4                        ; Joystick to left
       BRA   draw_cmd_ship2
PE2F2: PSHS  DP

 ;
 ; draw_cmd_ship2() runs the command ships coordinates
 ; thru a rotation transformation, and then draws the
 ; command ship using the transforms coordinates.
 ; draw_cmd_ship1() draws the command ship using the
 ; current transformed ship coordinates.
 ;
draw_cmd_ship2:
       JSR   rotate_cmd_ship              ; Rotate the command ship
draw_cmd_ship1:
       LDA   #$D0
       TFR   A,DP
       direct $D0
       JSR   intensity_to_5F
       LDB   #$0C
       LDY   #$C8C8                       ; Draw the xformed command ship
       LDX   #$CB89
       JSR   move_y_draw_x2
       direct $c8
PE30A: PULS  DP,PC
       direct $C8


; This routine handles the hyperspace request
hyperspace:
       LDA   #$80
       STA   $c8EE
       JSR   get_random_a
       ANDA  #$03
       ADDA  #$03
       STA   $c8EF
       INC   $c8F6
continue_hyperspace:
       LDA   $c8EE
       BPL   process_hyperspace_dots
       DEC   $c8EF
       BEQ   PE330
       JSR   $MS_get_2_random_nums
       STA   $c8C8
       CLR   $c8C9                        ; Generate new coordinates
       STB   $c8CA                        ; for the command ship.
       CLR   $c8CB
       direct $D0
       PULS  DP,PC
PE330: LSR   $c8EE
       LDA   #$1F
       STA   $c8EF
       direct $D0
       PULS  DP,PC


 ;
 ; This code draws the ever decreasing hyperspace dots
 ; during an active hyperspace.  The scale factor for
 ; the hyperspace dots, stored in C8EF, is decremented,
 ; and when it finally reaches a minimum value, the
 ; hyperspace is complete.
 ;
process_hyperspace_dots:
       LDB   $c8EF                        ; See if scale factor has reached
       CMPB  #$E0                         ; the limit.
       BLE   hyperspace_complete
       LDA   $c8EF
       SUBA  #$04                         ; Decrement the scale factor
       STA   $c8EF
       CLRA
       JSR   draw_hyperspace_motion_dots
       direct $D0
       PULS  DP,PC

hyperspace_complete:
       CLR   $c8EF
       CLR   $c8EE
       JSR   clear_cmd_ship_xformation
       direct $D0
       PULS  DP,PC


 ;
 ; After the player has cleared all but the small mines
 ; from a layer, the enemy ship will again appear, and
 ; attempt to reseed the screen.  This code checks to
 ; see if the enemy ship is visible (C8E7), and if it
 ; is, then it generates new coordinates for it, and
 ; then moves it.
 ;
reseed_screen:
       LDA   $C8E7
       BEQ   PE383                        ; Do only if ship is visible
       PSHS  DP
       LDA   #$C8
       TFR   A,DP
       direct $C8
       LDA   $c8E7
       BEQ   PE383
       LDD   $c8DE
       ADDD  $c8E2
       STD   $c8DE
       STA   $c8DC                        ; New y coordinate
       LDD   $c8E0
       ADDD  $c8E4
       STD   $c8E0
       STA   $c8DD                        ; New x coordinate
       PULS  DP
       direct $D0
       JSR   intensity_to_5F
       LDB   #$08
       LDY   $C8DC
       LDX   #simple_enemy_ship
       JSR   move_y_draw_x                ; Draw the enemy ship
PE383: RTS


 ;
 ; This procedure is invoked only through the indirect jump
 ; pointers.  After only small mines are left, this routine
 ; will eventually be called.  It will determine which way
 ; the enemy ship will move, and will also set up another
 ; indirect jump to the routine which does the actual
 ; reseeding of the screen.
 ;
ind_init_screen_reseeding:
       LDX   #ind_reseed_screen
       STX   $c8A3                        ; Set up next indirect jump ptr
       JSR    get_random_a
       LDX   #$E448                       ; Determine where enemy ship will move
       ANDA  #$06
       LDX   A,X
       LDD   ,X++
       STD   $c8DC                        ; Save enemy ships new coordinates
       STA   $c8DE
       CLR   $c8DF
       STB   $c8E0
       CLR   $c8E1
       BRA   PE3F9

 ;
 ; This routine is invoked only through the indirect jump
 ; mechanism.  It will continue to be called, until either
 ; the enemy ship is destroyed, or until the screen has
 ; been reseeded.  It will reseed upto 7 new mines.
 ;
ind_reseed_screen:
       LDA   $c8BF                        ; See if all mines have been seeded
       BNE   PE3BE
PE3A5: JSR   get_random_a
       ANDA  #$7F
       ADDA  #$30                         ; Set up the indirect jump counter 4
       STA   $c8A2                        ; the next pass.
       JSR   get_random_a
       ANDA  #$3F
       STA   $c8E6
       JSR   get_random_a
       ADDA  #$10
       STA   $c8E7
       BRA   PE407
PE3BE: LDA   $c8BD
       BNE   PE3A5

       ;
       ; Load the b register with the number of mines (28),
       ; and load the u register with the pointer to the
       ; mine structure.  Then look through the mine structure
       ; for an available slot, and fill it with the newly
       ; seeded mine.
       ;
       LDB   #$1C
       LDU   #$C933
PE3C7: LDA   ,U                           ; Look for an unused mine
       BEQ   found_avail_mine
       LEAU  $12,U
       DECB
       BNE   PE3C7
       BRA   PE407

found_avail_mine:
       INC   $c8ED                        ; Incr the # of mines which R dots
       DEC   $c8BF                        ; Decrement the reseed counter
       LDX   $c8DE
       STX   4,U                          ; Set y coordinate for mine
       LDX   $c8E0
       STX   6,U                          ; Set x coordinate for mine
       LDA   #$40
       STA   ,U                           ; Flag that mine is a stationary dot.
       LDA   $c8C0                        ; See if this is 1st mine activated.
       BNE   PE3F7
       LDX   #ind_start_large_mine;
       STX   $c89D                        ; Set things up to eventually
       JSR    get_random_a                ; activate 1 large mine.
       ANDA  #$7F
       ADDA  #$40
       STA   $c89C
       INC   $c8C0                        ; Flag that 1st mine has been activated
PE3F7: LDX   $c8E8
PE3F9: LDA   ,X+
       STA   $c8A2
       LDA   ,X+
       STA   $c8E6
       LDA   ,X+
       STA   $c8E7
       STX   $c8E8
PE407: LDB   $c8E6
       JSR   calc_rise_run1               ; Generate rise & run for mine
       STY   $c8E2
       STX   $c8E4
       RTS

 ;
 ; This routine is called only thru the indirect jump
 ; handler.  It is responsible for activating a single
 ; large mine.  The type of mine is indicated by the
 ; first entry in the table of mines for this level.
 ;
ind_start_large_mine:
       LDU   #$C8C4
       LDA   $c89B
       LDU   A,U
       LDA   ,U                           ; Determine the type of mine
       LDB   #$03;                        ; Set generation to 'largest'
       JSR   activate_a_mine
       LDX   #ind_activate_rangen_mine
       STX   $c89D
       RTS

 ;
 ; This routine decrements the counter in C8C1, and if
 ; it equal 0, then it activates a mine with a random
 ; generation number; else, it sets the indirect counter
 ; in C89C to -1, and returns.  This routine is called
 ; only thru the indirect jump handler.
 ;
ind_activate_rangen_mine:
       DEC   $c8C1
       BEQ   PE430
       LDA   #$FF
       STA   $c89C
       BRA   PE447
PE430: JSR   get_random_a
       TFR   A,B
       ANDB  #$03
       BNE   PE43B
       ADDB  #$01
PE43B: LDU   #$C8C4
       LDA   $c89B
       LDU   A,U
       LDA   ,U
       JSR   activate_a_mine
PE447: RTS


; Some sort of pointer table
SE448: DW    #SE450
       DW    #SE46A
       DW    #SE484
       DW    #SE49E

SE450: DB    $7F
       DB    $00
       DB    $28
       DB    $20
       DB    $30
       DB    $40
       DB    $28
       DB    $30
       DB    $28
       DB    $00
       DB    $10
       DB    $30
       DB    $10
       DB    $40
       DB    $18
       DB    $20
       DB    $50
       DB    $40
       DB    $30
       DB    $28
       DB    $30
       DB    $08
       DB    $60
       DB    $7F
       DB    $38
       DB    $70

SE46A: DB    $80
       DB    $00
       DB    $40
       DB    $00
       DB    $30
       DB    $20
       DB    $10
       DB    $50
       DB    $20
       DB    $28
       DB    $40
       DB    $30
       DB    $3E
       DB    $70
       DB    $18
       DB    $30
       DB    $60
       DB    $20
       DB    $18
       DB    $40
       DB    $30
       DB    $24
       DB    $50
       DB    $7F
       DB    $06
       DB    $70

SE484: DB    $00
       DB    $7F
       DB    $40
       DB    $10
       DB    $60
       DB    $28
       DB    $38
       DB    $30
       DB    $28
       DB    $08
       DB    $40
       DB    $30
       DB    $28
       DB    $7F
       DB    $20
       DB    $18
       DB    $30
       DB    $30
       DB    $08
       DB    $68
       DB    $40
       DB    $20
       DB    $50
       DB    $7F
       DB    $38
       DB    $70

SE49E: DB    $00
       DB    $80
       DB    $40
       DB    $30
       DB    $60
       DB    $38
       DB    $18
       DB    $30
       DB    $30
       DB    $20
       DB    $18
       DB    $20
       DB    $38
       DB    $40
       DB    $28
       DB    $10
       DB    $60
       DB    $20
       DB    $00
       DB    $30
       DB    $40
       DB    $38
       DB    $50
       DB    $7F
       DB    $1C
       DB    $70


 ;
 ; This routine is responsible for displaying any active
 ; bullets, and also for firing new bullets, when an
 ; available spot in the bullet structure becomes available.
 ; The maximum number of active bullets is 4.  The 40 byte
 ; bullet structure starts at C90B.  A bullet will remain
 ; 'active' for at most $18 passes thru this routine.
 ;
process_bullets:
       LDA   #$04                         ; Loop counter = 4
       LDU   #$C90B                       ; Bullet buffer
       LDX   #$C815                       ; State of fire button (4)
       STA   $C88F
       JSR   intensity_to_7F
next_bullet:
       LDA   ,U                           ; If the current spot is empty, then
       BEQ   check_fire_button            ; see if a new bullet should be fired
       DEC   9,U                          ; Decrement the cycle counter for the
       BEQ   erase_bullet                 ; and erase bullet if it = 0.
       LDD   5,U
       ADDD  1,U                          ; y
       STD   5,U                          ; Increment bullet coordinates
       LDD   7,U
       ADDD  3,U                          ; x
       STD   7,U
       LEAY  5,U
       JSR   draw_dot7F_ptr_in_y          ; Draw bullet
update_bufr_ptr:
       LEAU  10,U
       DEC   $C88F                        ; Point to next entry
       BNE   next_bullet
       RTS

erase_bullet:
       CLR   ,U
       DEC   $C8EA                        ; Decrement active bullet counter
check_fire_button:
       LDA   $C8BD
       BNE   update_bufr_ptr
       LDA   $C8EE
       BNE   update_bufr_ptr
       LDA   ,X                           ; Check the 'fire' button state
       BEQ   update_bufr_ptr

        ;
        ; This code 'fires' a new bullet, and fills the
        ; available spot in the bullet structure.
        ;
       CLR   ,X
       INC   $C8B6                        ; Flag that a bullet sound should be made
       INC   ,U
       LDD   $C8C8                        ; Get ships coordinates
       STD   5,U
       LDD   $C8CA
       STD   7,U
       LDD   $C907                        ; Get ship rise
       STD   1,U
       LDD   $C909                        ; Get ship run
       STD   3,U
       LDA   #$18                         ; # of passes before bullet expires
       STA   9,U
       INC   $C8EA                        ; Increment active bullet counter
       BRA   update_bufr_ptr

 ;
 ; This procedure processes each mine which has not been
 ; destroyed.  If the mine is a moving dot, then its position
 ; is updated; if the mine is a dot, then it is draw in its
 ; appropriate place; if the mine is active, then the appropriate
 ; mine is drawn; if it is a fireball, then the fireball is drawn.
 ; The state of the mine is determined by the first byte in the
 ; mine structure.  The following values are valid:
 ;
 ;      $00 - Mine is dead.
 ;      $01 - Mine is in process of being destroyed.
 ;      $08 - Mine is visible and moving.
 ;      $10 - Mine is visible, but not moving.
 ;      $20 - Mine is just becoming visible.
 ;      $40 - Mine is a stationary dot.
 ;      $80 - Mine is still moving to final resting place.
 ;
 ;
draw_non_dead_mines:
       LDA   #$1C
       STA   $C88F                        ; Use C88F as a loop counter
       LDU   #$C933                       ; Put mine pointer into u register
PE526: LDA   ,U
       BNE   non_dead_mine
update_mine_ptr:
       LEAU  $12,U
       DEC   $C88F
       BNE   PE526
       RTS

non_dead_mine:
       LBMI  move_mine_dot
       BITA  #$40
       LBNE  draw_mine_as_dot
       BITA  #$20
       LBNE  mine_becoming_visible
       BITA  #$10
       LBNE  delay_B4_moving_mine
       BITA  #$01
       LBNE  process_destroyed_mine
       LDA   1,U
       CMPA  #$04                         ; See if this is a fireball
       BEQ   process_fireball
       BITA  #$01                         ; Code below moves visible mine
       BEQ   non_magnetic_mine
       LDA   $C8EE                        ; Skip below if hyperspace active
       BNE   non_magnetic_mine
       LDA   $C8BD                        ; Skip below if cmd ship disabled
       BNE   non_magnetic_mine
       PSHS  DP
       JSR   dptoC8
       direct $c8
       LDA   $C8                          ; Draw the mine towards the cmd ship
       SUBA  4,U
       LDB   $c8CA
       SUBB  6,U
       JSR   $PF593
       SUBA  #$10
       STA   $c883
       LDX   #mine_velocity
       LDB   3,U
       LDA   B,X
       LDB   $c883
       JSR   calc_rise_run1               ; Calculate new rise and run values
       STY   8,U
       STX   10,U
       PULS  DP
       direct $D0
non_magnetic_mine:
       LDD   4,U                          ; This block update mine coordinates
       ADDD  8,U
       STD   4,U
       LDD   6,U
       ADDD  10,U
       STD   6,U

 ;
 ; This block of code draws a mine, using information in
 ; the mine structure element pointed to by the u register.
 ;
draw_a_visible_mine:
       JSR   intensity_to_5F
       LDX   #mine_vector_ptrs
       LDA   1,U                          ; byte 1 = mine type
       ASLA
       LDX   A,X                          ; Set x = ptr to mine vector list
       LEAY  4,U                          ; Set y = mine coordinates.
       LDB   2,U                          ; Set b = scale factor.
       JSR   move_y_draw_x2
       JMP   update_mine_ptr

 ;
 ; This routine appears to be processing a fireball.
 ; It checks to see if the fireball has reached the
 ; edge of the display, and if it has, then the
 ; fireball is removed; else, the fireball is drawn
 ; on the display.
 ;
process_fireball:
       LDD   4,U                          ; See if the fireball has
       ADDD  8,U                          ; reached the edge of the
       BVS   remove_fireball              ; display.
       STD   4,U
       LDD   6,U
       ADDD  10,U
       BVS   remove_fireball
       STD   6,U
       JSR   intensity_to_7F
       LEAY  4,U                          ; Draw the fireball
       LDX   #$CBA7
       LDB   #$04
       JSR   move_y_draw_x2
       JMP   update_mine_ptr

remove_fireball:
       CLR   ,U                           ; Flag that the fireball has
       DEC   $C8EB                        ; been removed.
       JMP   update_mine_ptr

 ;
 ; move_mine_dot () does some sort of processing on the
 ; mine's coordinates, and then draws the mine as
 ; a dot.  I believe that the processing being done
 ; is the movement of the dot in the x direction,
 ; after the mine field has been seeded, but before
 ; the mine dot has reached it's final place.
 ; draw_mine_as_dot() draws a mine as a dot.
 ;
 ; For both these routines, the u register must
 ; point to a structure element describing the
 ; mine.
 ;
move_mine_dot:
       LDA   6,U                          ; Update the mines x coordinate,
       ADDA  $10,U                        ; and check to see if it has reached
       STA   6,U                          ; its final resting place.
       CMPA  $11,U
       BNE   draw_mine_as_dot             ; Flag that the mine dot is at its
       LSR   ,U                           ; final resting place.
draw_mine_as_dot:
       JSR   intensity_to_5F
       LEAY  4,U
       JSR   draw_dot7F_ptr_in_y
       JMP   update_mine_ptr


 ;
 ; This procedure is called when a mine first becomes visible.
 ; If the mine is not a large mine, then the mine is drawn at
 ; full scale factor; otherwise, if the mine is a large mine,
 ; then it will be drawn with a steadily increasing scale factor.
 ; This gives the effect of the mine 'popping' up.
 ;
mine_becoming_visible:
       LDA   3,U
       CMPA  #$03                         ; See if this is a large mine
       BNE   use_max_scale_factor
       LDA   2,U
       CMPA  $10,U
       BGE   use_max_scale_factor
       ADDA  #$08
       STA   2,U                          ; Increment the scale factor
       BRA   PE61A
use_max_scale_factor:
       LSR   ,U                           ; Flag that mine is full size & visible
       LDA   $10,U
       STA   2,U                          ; Set final scale factor
       LDA   #$18
       STA   $10,U                        ; Set up a delay
       LDA   $C8ED
       BNE   PE61A                        ; Check to see if any more mine dots left
       LDA   $C8C0
       BNE   PE61A                        ; See if re-seeding already in progress.
       LDA   #$7F                         ; Set up indirect counter for reseeding
       STA   $C8A2                        ; since only small mines are left.
PE61A: JMP   draw_a_visible_mine


 ;
 ; This procedure is used to delay the movement of a mine
 ; right after it become visible.  After the delay is over,
 ; the mine will start moving.
 ;
delay_B4_moving_mine:
       DEC   $10,U                        ; Decrement delay counter
       BNE   PE624
       LSR   ,U
PE624: JMP   draw_a_visible_mine


 ;
 ; This procedure processes the activation of 2 new mines
 ; after a mine has been destroyed.  If the destroyed mine
 ; was a small mine, or a fireball, then 2 new mines will
 ; not be activated.  At entry time, the 'a' register must
 ; contain the type of mine to be started up.
 ;
process_destroyed_mine:
       CLR   ,U                           ; Flag that this mine is gone
       LDA   1,U
       CMPA  #$04                         ; If this was a fireball, then don't
       BEQ   PE644                        ; start up 2 new mines.
       LDB   3,U
       DECB                               ; If this was a small mine, then don't
       BEQ   PE644                        ; start up 2 new mines.
       PSHS  A,DP
       LDA   #$C8
       TFR   A,DP
       direct $c8
       LDA   ,S                           ; Get the type of mine
       JSR   activate_a_mine
       JSR   activate_a_mine
       PULS  A,DP
       direct $D0
PE644: JMP   update_mine_ptr


 ;
 ; This routine is responsible for displaying the explosion
 ; pattern for all active explosions.  If the command ship
 ; is sexploding, then it will also switch over to the
 ; next player (eventually).
 ;
display_explosion_pattern:
       PSHS  DP
       JSR   dptoD0
       direct $D0
       JSR   intensity_to_7F
       LDU   #$CB2B                       ; Get pointer to the explosion struct
       LDA   #$0E
       STA   $C88F                        ; Use C88F as a loop counter
check_next_explosion:
       LDA   ,U
       LBEQ  inc_expl_ptr
       LDB   4,U                          ; If the scale factor for the explosion
       CMPB  1,U                          ; has not surpassed the max value, then
       BHS   chk_4_cmd_ship_explosion     ; increment it.
       ADDB  #$03
       STB   4,U
       LDY   2,U
       LDX   #explosion                   ; Draw the explosion pattern
       JSR   move_y_draw_x

chk_4_cmd_ship_explosion:
       TSTA                               ; This is a cmd ship explosion if bit
       LBPL  check_4_expired_explosion    ; $80 is set.
       DEC   $C8F7                        ; Stop displaying explosion when C8F7
       LBEQ  dead_ship                    ; reaches zero.
       LDA   $C826
       ANDA  #$01                         ; Every other pass through the loop,
       BNE   draw_expl_cmd_ship           ; increment the scale factor used to
       INC   $C8F8                        ; draw the destroyed command ship.

draw_expl_cmd_ship:
       LDA   $C8F8                        ; Draw the cmd ship in pieces.
       LDY   #$7F00
       LDX   #cmd_ship_pt1
       JSR   draw_cmd_ship_fragment
       LDY   #$6080
       LDX   #cmd_ship_pt2
       JSR   draw_cmd_ship_fragment
       LDY   #$8050
       LDX   #cmd_ship_pt3
       JSR   draw_cmd_ship_fragment
       LDY   #$A080
       LDX   #cmd_ship_pt4
       JSR   draw_cmd_ship_fragment
       BRA   inc_expl_ptr

 ;
 ; This routine is called after a ship has been destroyed.
 ; The number of ships for the active player, kept in C8D9,
 ; will be decremented.  If more than one player is playing,
 ; then save the active players ship count in a memory
 ; location (C8DA for player 1, and C8DB for player 2),
 ; and then determine who's turn it now is.
 ;
dead_ship:
       DEC   $C8D9                        ; Decrement the number of ships
       CLR   $C8EB
       CLR   $C8ED

       LDA   $C879                        ; If more than 1 player is playing,
       BEQ   check_ship_count             ; then save the active players ship
       LDA   $C89B                        ; ship count in RAM.
       LSRA
       LDX   #$C8DA
       LDB   $C8D9
       STB   A,X

       LDA   $C8DA                        ; See who's turn it is.  Skip a player
       BNE   next_player                  ; if he has no more ships.
       LDA   $C8DB
       BEQ   flag_no_ships_left

next_player:
       LDA   $C89B                        ; Increment active player indicator
       ADDA  #$02
       ANDA  #$02
       STA   $C89B

       LSRA                               ; Load the ship count for new player
       LDX   #$C8DA
       LDB   a,x
       STB   $C8D9
       BEQ   next_player

 ;
 ; This routine checks to see if the active player has any ships
 ; left, after the command ship has exploded.
 ;
check_ship_count:
       LDA   $C8D9
       BNE   explosion_expired
flag_no_ships_left:
       LDA   #$01
       STA   $C8BE                        ; Set C8BE when no ships left
       BRA   explosion_expired

 ;
 ; This routine checks to see if the explosion has expired.
 ; An explosion expires when its scale factor surpasses a
 ; preset maximum value.
 ;
check_4_expired_explosion:
       LDB   4,U                          ; See if scale factor has reached
       CMPB  1,U                          ; the maximum value allowed.
       BLO   inc_expl_ptr
explosion_expired:
       CLR   ,U
       DEC   $C8EC                        ; Flag completion of cmd ship explosion

inc_expl_ptr:
       LEAU  5,U
       DEC   $C88F
       LBNE  check_next_explosion
       JSR   make_misc_sounds
       BRA   draw_tiny_ships

; Draw the tiny ships in lower righthand corner
PE711: PSHS  DP
       JSR   dptoD0
       direct $D0
draw_tiny_ship:
       JSR   intensity_to_5F
       LDX   #$8038
       STX   $C890                        ; Position for first ship
       LDA   $C8D9                        ; Don't bother, if the active
       BEQ   rotate_fireball_vectors      ; player has no ships left.
       STA   $C88F
PE727: DEC   $C88F
       BEQ   rotate_fireball_vectors      ; Exit when all ships drawn
       LDA   $C891
       ADDA  #$06                         ; Increment x coordinate
       STA   $C891
       LDB   #$04
       LDY   $C890
       LDX   #command_ship
       JSR   move_y_draw_x                ; Draw the tiny command ship
       BRA   PE727

 ;
 ; This routine rotates the fireball vector list.  Also,
 ; it checks to see whether this level has been cleared.
 ;
rotate_fireball_vectors:
       PULS  DP
       direct $C8
       LDA   $c826
       ANDA  #$01
       ASLA
       ASLA
       ASLA
       LDX   #fireball
       LDU   #$CBA7
       JSR   rotate_vector_list1

       ;
       ; This block of code determines whether or not the main
       ; loop should be exited.  If the cmd ship is disabled, or
       ; if the player has cleared the level, then the main loop
       ; will be exited (this is flagged by clearing the carry bit.
       ;
       LDB   $c8EC                        ; See if cmd ship is being exploded
       BNE   PE767
       LDA   $c8BD                        ; See if cmd ship is disabled
       BNE   PE764
       LDB   $c8EB                        ; check # of active mines
       BNE   PE767
       LDB   $c8ED                        ; check # of dot mines
       BNE   PE767
PE764: ANDCC #$FE                         ; Says 'exit main loop'
       RTS
PE767: ORCC  #$01                         ; Says 'do not exit main loop'
       RTS


 ;
 ; This routine is responsible for drawing a single fragment
 ; of the exploding command ship.  At entry, 'a' = scale factor,
 ; 'y' = pen position, and 'x' = ptr to vector list.
 ;
draw_cmd_ship_fragment:
       PSHS  A,X,Y;
       ldx   #$C8C8                       ; Move pen to cmd ships location
       jsr   move_pen7F_no_inc
       LDA   ,S
       STA   $c804                        ; Set scale factor = a register
       TFR   Y,D
       JSR   move_pen_d                   ; Move pen to position in y register
       LDB   #$0C
       LDX   1,S
       JSR   drawl1b                      ; Draw vector list pointed to by x reg
       PULS  A,X,Y,PC


 ;
 ; This routine is called after a bullet has hit something,
 ; or the command ship collides with something.  It will
 ; search through the explosion structure, looking for an
 ; empty slot.  When it finds one, it will fill it with the
 ; information passed in.  At entry time, 'a' = type of explosion
 ; and the scale factor to use, 'b' = the max scale factor to
 ; be used, and 'x' contains the position for the explosion.
 ; If the hi bit of 'a' is set, then that flags that it is the
 ; command ship exploding; the lower 7 bits are used to specify
 ; the initial scale factor.
 ;
add_an_explosion:
       PSHS  A,B,X
       LDX   #$CB2B
       LDA   #$0E
PE78B: LDB   ,X                           ; Find an empty spot in structure
       BEQ   found_empty_exp_slot
       LEAX  5,X
       DECA
       BNE   PE78B
       BRA   PE7B3

found_empty_exp_slot:
       LDA   ,S                           ; Save the explosion type
       ANDA  #$80
       INCA
       STA   ,X                           ; See if this is a cmd ship explosion
       BPL   PE7A1
       INC   $c8BD                        ; Flag that cmd ship is disabled
PE7A1: LDA   ,S
       ANDA  #$7F
       STA   4,X                          ; Save starting scale factor
       LDA   1,S
       STA   1,X
       LDD   2,S
       STD   2,X                          ; Save maximum scale factor
       INC   $c8EC                        ; Flag that cmd ship is exploding
       INC   $c8F3                        ; Flag that explosion sound should be made
PE7B3: PULS  A,B,X,PC


 ;
 ; This routine is responsible for calculating a rise and run
 ; for an object whose angle of travel is in the 'b' register,
 ; and whose velocity is in the 'a' register.
 ; The rise is returned in the 'y' register, and the run is
 ; returned in the 'x' register.
 ;
calc_rise_run1:
       PSHS  A,B,X,Y
       JSR   rotate_1_coordinate
       STA   4,S
       SEX
       ASLB
       ROLA
       ASLB
       ROLA
       ASLB
       ROLA
       STD   2,S
       LDB   4,S
       SEX
       ASLB
       ROLA
       ASLB
       ROLA
       ASLB
       ROLA
       STD   4,S
       PULS  A,B,X,Y,PC


 ;
 ; This routine also calculates some sort of rise & run pair.
 ; The same parameters are expected at entry time, and the
 ; values are returned in the same registers.
 ;
calc_rise_run2:
       PSHS  A,B,X,Y
       BSR   calc_rise_run1
       LDD   -4,S
       ASLB
       ROLA
       STD   4,S
       LDD   -6,S
       ASLB
       ROLA
       STD   2,S
       PULS  A,B,X,Y,PC

 ;
 ; This routine is called each time a new mine level is
 ; started.  It initializes everything necessary.
 ;
init_RAM_for_new_level:
       LDA   #$D0
       TFR   A,DP
       direct $D0
       jsr   clear_sound_chip
       LDA   #$C8
       TFR   A,DP
       direct $C8
       CLR   $c89C                        ; Clear all 4 indirect jump counters
       CLR   $c89F
       CLR   $c8A2
       CLR   $c8A5
       LDX   #$C90B                       ; Clear C90B - CB71
PE7FA: CLR   ,X+
       CMPX  #$CB71
       BNE   PE7FA
       LDD   #$0000
       STD   $c8DE
       STD   $c8E0
       STD   $c8E2
       STD   $c8E4
       STA   $c8E7
       STA   $c8BD                        ; Clear cmd ship disabled flag
       STA   $c8BE                        ; Clear 'no more ships' flag
       STA   $c8EA                        ; Clear active bullet counter
       STA   $c8EB
       STA   $c8EC                        ; Clear cmd ship exploding flag
       STA   $c8F8                        ; Clear destroyed cmd ship scale factor
       LDB   #$40
       STB   $c8F7                        ; Init cmd ship exploding loop counter
       STA   $c8ED                        ; Clear # of dot mines
       STA   $c8C0                        ; Clear mine activated during reseeding flag
       LDX   #$0800
       STX   $c8F0                        ; Set end of game loop counter
       LDA   #$07
       STA   $c8BF                        ; Init # of mines to be reseeded
       ldx   #ind_init_screen_seeding
       STX   $c8A3                        ; Init indirect jump location for reseeding
       LDD   #$0000
       STD   $c8C8                        ; Clear the command ship's
       STD   $c8CA                        ; position indicators.

clear_cmd_ship_xformation:
       LDD   #$0000
       STA   $c8D4                        ; Clear rotation counter
       STD   $c8CC
       STD   $c8CE
       STA   $c8D5                        ; Clear cmd ships velocity
       STA   $c8D6
       STD   $c8D0
       STD   $c8D2
       STA   $c8D7
       STA   $c8D8

 ;
 ; This block of code takes the vector list describing the
 ; command ship, and applies a rotation transformation to
 ; them.  The current rotation value is in C8D4, and the
 ; transformed vector list is stored in the buffer
 ; starting at CB89.
 ;
rotate_cmd_ship:
       LDA   $c8D4
       LDX   #command_ship
       LDU   #$CB89
       JSR   rotate_vector_list1

       LDA   #$7F
       LDB   $c8D4
       JSR   calc_rise_run2
       STY   $C907
       STX   $C909
       RTS


 ;
 ; This routine is called after the active player clears
 ; a mine field level.  This routine will slowly bring the
 ; command ship back to the center of the screen.  As this
 ; is being done, a series of dots will be drawn to simulate
 ; motion.
 ;
bring_ship_to_center:
       PSHS  X,Y
       PSHS  DP
       JSR   dptoD0
       direct $D0
       JSR   clear_sound_chip
       PULS  DP
       direct $C8
       LDA   #$A0
       STA   $c88F                        ; Use C88F as a loop counter

PE876: LDA   $c8C8
       BEQ   PE884                        ; Modify the y coordinate for the
       BMI   PE87F                        ; command ship, in an effort to
       DECA                               ; force it back to 0.
       BRA   PE880
PE87F: INCA
PE880: STA   $c8C8
       CLR   $c8C9

PE884: LDA   $c8CA                        ; Modify the x coordinate for the
       BEQ   PE892                        ; command ship, in an effort to
       BMI   PE88D                        ; force it back to 0.
       DECA
       BRA   PE88E
PE88D: INCA
PE88E: STA   $c8CA
       CLR   $c8CB

PE892: LDA   $c8D4                        ; Modify the rotation value for the
       BEQ   PE8A2                        ; command ship, in an effort to
       CMPA  #$1F                         ; force it back to 0.
       BGT   PE89D
       DECA
       BRA   PE89E
PE89D: INCA
PE89E: ANDA  #$3F
       STA   $c8D4

PE8A2: JSR   $PE2F2                       ; This loop, repeated 8 times (once
       LDX   #$CB81                       ; for each scale factor in the array)
       LDB   #$08                         ; increments each of the scale factors
PE8AA: LDA   ,X                           ; for the motion dots, by 3.
       ADDA  #$03
       STA   ,X+
       DECB
       BNE   PE8AA

       PSHS  DP
       JSR   dptoD0
       direct $D0
       JSR   display_both_scores
       CLRB;
       LDA   #$20                         ; " "
       JSR   simulate_motion1
       JSR   modify_motion_scales
       PULS  DP
       direct $C8

       ; Repeat the above until the ship is centered.
       LDA   $c8C8
       LBNE  PE876
       LDA   $c8CA
       LBNE  PE876
       LDA   $c8D4
       LBNE  PE876
       DEC   $c88F
       LBNE  PE876
       JSR   init_RAM_for_new_level
       PULS  X,Y,PC


 ;
 ; This routine initializes the two arrays
 ; (CB71 and CB81) which contain the data
 ; needed to produce the motion dots (the
 ; dots displayed as the ship moves from
 ; one level to the next).  The 8 entries
 ; in CB71 are pointers to the 8 sets of
 ; required dots.  The 8 entries in CB81
 ; are the corresponding scale factors.
 ;
init_motion_dots:
       LDX   #motion_dots
       LDY   #$CB71
       LDU   #$CB81
       LDB   #$08
       LDA   #$16
PE8F1: STX   ,Y++
       LEAX  8,X
       STA   ,U+
       ADDA  #$0F
       DECB
       BNE   PE8F1
       RTS

 ;
 ; modify_motion_scales() increments each of the 8 motion
 ; scale factors, and then draws the new motion dots.
 ; simulate_motion1() and simulate_motion2() both
 ; draw the current motion dots.  The motion dots will
 ; only be drawn if the scale factor is > the value specified
 ; in the 'a' register, and if (scale - 'b') > 0.
 ;
modify_motion_scales:
       PSHS  A,B,DP,X
       LDX   #$CB81
       LDA   #$08
PE904: INC   ,X+                          ; Increment each of the scale factors ;
       DECA
       BNE   PE904
       BRA   simulate_motion2

simulate_motion1:
       PSHS  A,B,DP,X
simulate_motion2:
       LDA   #$D0
       TFR   A,DP
       direct $D0
       LDA   #$09                         ; Use 'a' as a loop counter
       PSHS  A
PE915: DEC   ,S
       BNE   PE920
       JSR   reset0ref
       PULS  A
       PULS  A,B,DP,X,PC
       direct $D0
PE920: JSR   $reset0ref
       LDA   #$03                         ; Use C823 to hold # of dots
       STA   $C823                        ; to be drawn (4).
       LDA   ,S
       DECA
       LDX   #$CB81
       LDB   A,X
       ANDB  #$7F
       CMPB  1,S                          ; Make sure scale factor is
       BLS   PE915                        ; within the specified range
       SUBB  2,S
       BLE   PE915
       STB   $04                          ; Set the hardware scale factor

       LDX   #$CB71                       ; Draw the dot list
       ASLA                               ; Convert loop cntr to a word offset
       LDX   A,X
       JSR   intensity_to_7F
       JSR   dot_list
       BRA   PE915


 ;
 ; This routine is responsible for drawing the hyperspace dot
 ; during a hyperspace.  It draws the 8 sets of motion dots,
 ; using a decreasing scale factor.  At entry, the 'b' register
 ; contains the minimum scale factor to be used.
 ;
draw_hyperspace_motion_dots:
       PSHS  A,B,DP,X
       LDA   #$D0
       TFR   A,DP
       direct $D0
       LDA   #$09                         ; Push the loop cntr on the stack
       PSHS  A
next_set_of_dots:
       DEC   ,S
       BNE   draw_the_dots
       JSR   reset0ref
       PULS  A
       PULS  A,B,DP,X,PC
       direct $D0

draw_the_dots:
       JSR   reset0ref
       LDA   #$03
       STA   $C823                        ; Use C823 to hold # of dots to draw
       LDX   #$C8C8
       JSR   move_pen7F_no_inc
       LDB   ,S
       ASLB
       ASLB
       ADDB  2,S                          ; Calculate the scale factor
       BLE   next_set_of_dots
       ANDB  #$7F
       STB   $D004                        ; Set scale factor
       LDX   #$CB71
       LDA   ,S                           ; Draw the appropriate set of dots
       DECA
       ASLA
       LDX   A,X
       JSR   intensity_to_7F
       JSR   dot_list
       BRA   next_set_of_dots


 ;
 ; Generate 2 random numbers (1 byte each), and return
 ; them in a and b.  The range for the two numbers is:
 ;
 ;      $80 <= first #  <= $7F     (a register)
 ;      $A0 <= second # <= $60     (b register)
 ;
MS_get_2_random_nums:
       PSHS  A,B
       JSR   get_random_a
       STA   ,S
PE991: JSR   get_random_a
       CMPA  #$60
       BGT   PE991
       CMPA  #$A0
       BLT   PE991
       STA   1,S
       PULS  A,B
       RTS;

 ;
 ; This routine randomly activates a single mine.
 ; At entry, the 'a' register must contain the
 ; type of mine to be activated, while the 'b'
 ; register must contain the generation number
 ; for the mine (i.e. large, medium, or small).
 ;
activate_a_mine:
       PSHS  A,B,X,Y,U
       LDA   $c8ED                        ; Check C8ED to see if there are any
       LBEQ  PEA3C                        ; dot mines; if not, then return.
       DEC   $c8ED

       JSR   get_random_a                 ; Generate a random index into the
       ANDA  #$1F                         ; mine array.
PE9B0: STA   $c88B
       CMPA  #$1B
       BLS   PE9BA
       SUBA  #$04
       BRA   PE9B0

PE9BA: LDB   #$12                         ; Multiple offset by element size
       MUL
       ADDD  #$C933
       TFR   D,U
       LDA   ,U                           ; Check to see if this mine is already
       ANDA  #$C0                         ; being used.  If it is, then check
       BNE   init_a_visible_mine          ; the next mine in the structure, else
       INC   $c88B                        ; use this mine.
       LDA   $c88B
       CMPA  #$1B
       BLE   PE9BA
       CLR   $c88B
       CLRA
       BRA   PE9BA
init_a_visible_mine:
       LDA   ,S                           ; Store the mine type
       STA   1,U
       ldx   #mine_values
       ASLA
       LDY   A,X
       STY   $c889                          ; Store mine's value in C889
       LDB   #$20
       STB   ,U
       LDX   #mine_velocity
       LDA   1,S                          ; Save mine's velocity
       LDB   A,X
       STB   $C88B
       ldx   #final_mine_scale_factor
       LDB   A,X
       STB   $10,U                        ; Save mine's max scale factor
       STA   3,U                          ; Save mine's generation number
       LDX   #mine_sizes
       ASLA
       LDY   A,X
       STY   $12,U                        ; Save mine's size
       LDX   #extra_mine_value
       LDY   A,X
       STY   $C887                        ; Get any extra value for mine's size
       CMPA  #$06
       BNE   PEA13
       INC   $C8F4                        ; Flag that a sound should be made
PEA13: LDA   $C888
       ADDA  $C88A
       DAA
       STA   $15,U                        ; Generate lo byte of score
       LDA   $C887
       ADCA  $C889
       DAA
       STA   $14,U                        ; Generate hi byte of score
       LDA   $C88B
       JSR   MS_random_num_4_to_3C
       JSR   calc_rise_run1
       STY   8,U                          ; Save the rise
       STX   10,U                         ; Save the run
       INC   $C8EB                        ; Increment 'active mine' counter
       LDA   $C8C0
       BEQ   PEA3C
       LDA   #$FF
       STA   $C89C
       LDA   #$03
       STA   $C8C1
PEA3C: PULS  A,B,X,Y,U,PC


 ;
 ; Generate a random number between 4 and $3C.  The
 ; result is returned in the b register.
 ;
MS_random_num_4_to_3C:
       PSHS  A,B
       JSR   get_random_a
       TFR   A,B
       ANDA  #$30                         ; "0"
       STA   1,S
       ANDB  #$0F
       CMPB  #$04
       BHS   PEA51
       ADDB  #$04
PEA51: CMPB  #$0C
       BLS   PEA57
       SUBB  #$04
PEA57: ADDB  1,S
       STB   1,S
       PULS  A,B,PC

 ;
 ; Force the scale factor to $7F, then
 ; draw a dot at the position contained
 ; in the y register.
 ;
draw_dot7F_at_y:
       PSHS  A,B
       LDA   #$7F
       STA   $D004
       TFR   Y,D
       JSR   dot
       JSR   reset0ref
       PULS  A,B,PC

 ;
 ; Force the scale factor to $7F, then draw a
 ; dot at the position pointed to by the y
 ; register (y coord = (0,y), x coord = (2,y)).
 ;
draw_dot7F_ptr_in_y:
       PSHS  A,B
       LDA   #$7F
       STA   $D004
       LDA   ,Y
       LDB   2,Y
       JSR   dot
       JSR   reset0ref
       PULS  A,B,PC;


 ;
 ; Move the pen to the position specified
 ; in the y register, then draw the vector
 ; list pointed to by the x register; the
 ; scale factor is in the b register.
 ;
move_y_draw_x:
       PSHS  A,B,X
       TFR   Y,D
       JSR   move_pen7F_to_d
       LDB   1,S
       JSR   drawl1b
       PULS  A,B,X,PC


 ;
 ; Move the pen to the position specified in the y
 ; register, and draw the list of vectors pointed to
 ; by the x register; the scale factor to be used is
 ; in the b register. (y coord = (0,y), x coord = (x,y))
 ;
move_y_draw_x2:
       PSHS  A,B,X
       TFR   Y,X
       JSR   move_pen7F_no_inc
       LDB   1,S
       LDX   2,S
       JSR   drawl1b
       PULS  A,B,X,PC

 ;
 ; Force the scale factor to $7F, then print
 ; a single string (upto a $80).  The pointer
 ; to the string block is contained in the
 ; u register.
 ;
MS_print_1_string7F:
       PSHS  A,B,X,U
       LDA   #$7F
       STA   $D004
       JSR   print_1_string
       PULS  A,B,X,U,PC

 ;
 ; Move the pen to the position specified in the
 ; y register, then print a string.  The pointer
 ; to the string (not a string block) is in the
 ; u register.
 ;
move_y_print_string:
       PSHS  A,B,X,U
       TFR   Y,D
       JSR   move_pen7F_to_d
       JSR   display_string
       PULS  A,B,X,Y,PC


 ;
 ; Use the value in C89B (the active player) to
 ; determine which players score to display,
 ; and then display the score string.
 ;
display_active_players_score:
       JSR   intensity_to_7F
       LDD   #$FC38
       STD   $C82A                        ; Specify height and width
       LDA   $C89B
       LDY   #score_string_positions
       LDY   A,Y                          ; Load string position into y
       LDU   #score_string_pointers
       LDU   A,U                          ; Load score string into u
       BSR   move_y_print_string
       RTS

 ;
 ; Display the score for both players
 ;
display_both_scores:
       JSR   intensity_to_7F
       LDD   #$FC38
       STD   $C82A                        ; Specify height and width
       LDY   #$7FA0                       ; Load y with y&x positions
       LDU   #$C8A8                       ; Load u with player 1 score
       BSR   move_y_print_string
       LDA   $C879                        ; See if we need to display player 2 score
       BEQ   PEAEF
       LDY   #$7F10                       ; Load y with y&x positions
       LDU   #$C8AF                       ; Load u with player 2 score
       BSR   move_y_print_string
PEAEF: RTS;


 ;
 ; This routine is responsible for reading the current
 ; states of the joystick and buttons.  Next, it will
 ; check each of the four indirect jump counters, and
 ; if any of them equal 0 after being decremented, then
 ; the appropriate indirect handling routine will be
 ; called.  The four indirect counters, and their corresponding
 ; handler routines are shown below:
 ;
 ;         counter          handler
 ;         -------          -------
 ;           C89C           C89D-C89E
 ;           C89F           C8A0-C8A1
 ;           C8A2           C8A3-C8A4
 ;           C8A5           C8A6-C8A7
 ;
 ;
process_indirect_jumps:
       JSR   waitrecal
       direct D0
       PSHS  DP
       JSR   $PF2E6
       JSR   display_active_players_score

       ; SET mask (in C880) and read the switches
       LDA   $C880
       JSR   read_switches

       ; Set enable flags (in C881) and read joystick
       LDD   $C881
       STD   $C81F
       STD   $C821
       JSR   read_jstick

       LDA   #$C8
       TFR   A,DP
       direct $C8
       LDA   $C89C
       BEQ   PEB1D
       DEC   $C89C
       BNE   PEB1D
       JSR   [C89D]                       ; (UNKNOWN JUMP)
PEB1D: LDA   $C89F
       BEQ   PEB29
       DEC   $C89F
       BNE   PEB29
       JSR   [C8A0]                       ; (UNKNOWN JUMP)
PEB29: LDA   $C8A2
       BEQ   PEB35
       DEC   $C8A2
       BNE   PEB35
       JSR   [C8A3]                       ; (UNKNOWN JUMP)
PEB35: LDA   $C8A5
       BEQ   PEB41
       DEC   $C8A5
       BNE   PEB41
       JSR   [C8A6]                       ; (UNKNOWN JUMP)
       direct $D0
PEB41: PULS  DP,PC


 ;
 ; This routine check to see if any of the
 ; bullets have hit a mine or a fireball or
 ; the enemy ship.  If something is hit, then
 ; the active player's score will be modified.
 ;
check_bullets_for_hits:
       LDA   $C8EA                        ; Don't bother checking if there are
       BEQ   PEB59                        ; no active bullets. (C8EA = 0)
       LDY   #$C90B
       LDA   #$04
       STA   $C88F                        ; Use C88F as a loop counter
check_for_active_bullet:
       TST   ,Y                           ; See if this bullet is active
       BNE   check_enemy_ship_4_hit
inc_bullet_ptr2:
       LEAY  10,Y                         ; Increment ptr to next bullet
       DEC   $C88F
       BNE   check_for_active_bullet
PEB59: RTS

check_enemy_ship_4_hit:
       LDA   $E7                          ; Only check for a hit on the enemy
       BEQ   check_mines_for_hits         ; ship if it is showing (C8E7 > 0).
       PSHS  Y
       LDA   5,Y
       LDB   7,Y
       TFR   D,X                          ; bullets position
       LDD   #$0616                       ; height/2 and width/2 of enemy ship
       LDY   $C8DC                        ; position of enemy ship
       JSR   check_bullet_for_hit         ; See if the enemy ship was hit
       PULS  Y
       BHS   check_mines_for_hits
       CLR   ,Y                           ; HIT - flag that bullet is dead
       CLR   $C8E7                        ; Flag that enemy ship no longer there
       CLR   $C8A2                        ; Clear 1 indirect jump counter
       LDX   #score_string_pointers
       LDA   $C89B
       LDX   A,X
       LDD   #$1000                       ; Inc players score by 1000
       JSR   add_d_to_x_in_bcd
       LDA   #$30
       LDB   #$70
       LDX   $C8DC
       JSR   add_an_explosion
       DEC   $C8EA                        ; Decrement the # of active bullets
       BRA   PEB59

 ;
 ; This routine checks to see if a bullet hit
 ; any of the active mines or fireballs.
 ;
check_mines_for_hit:
       LDU   #$C933                       ; Load 'u' with ptr to mine info
       LDA   #$1C
       STA   $C890                        ; Use C890 as a counter
check_for_active_mine:
       LDA   ,U
       ANDA  #$3F                         ; Make sure mine is active
       BNE   check_this_mine
inc_mine_ptr2:
       LEAU  $12,U                        ; Inc ptr into mine structure
       DEC   $90
       BNE   check_for_active_mine
       BRA   inc_bullet_ptr2

check_this_mine:
       PSHS  Y
       LDA   5,Y
       LDB   7,Y
       TFR   D,X                          ; bullet coordinates
       LDA   4,U
       LDB   6,U                          ; mine coordinates
       TFR   D,Y
       LDD   12,U                         ; height & width of mine
       JSR   check_bullet_for_hit         ; returns: carry set = hit, else miss
       PULS  Y
       BHS   inc_mine_ptr2
       LDA   1,U                          ; Check if this is a fireball mine
       ANDA  #$02
       BEQ   not_a_fireball_mine
       LDX   #score_string_pointers
       LDA   $C89B
       LDX   A,X
       LDD   14,U                         ; Add mines value to players score
       JSR   add_d_to_x_in_bcd
       INC   $C8F5
       LDA   4,U
       LDB   6,U
       TFR   d,X
       LDA   2,U
       LDB   #$20
       JSR   add_an_explosion
       LDD   #$0110                       ; Init score for the fireball
       STD   14,U
       LDA   $C8C8
       SUBA  4,U
       LDB   $C8CA
       SUBB  6,U
       JSR   PF593
       SUBA  #$10
       TFR   A,B
       PSHS  Y
       LDA   #$3F
       JSR   calc_rise_run1
       STY   8,U                          ; Calculate rise & run for fireball
       STX   10,U
       PULS  Y
       CLR   ,Y                           ; Flag that the bullet is dead
       LDD   #$0404
       STD   12,U                         ; Set height/2 & width/2 for fireball
       LDA   1,U
       LDB   3,U                          ; Decrement the generation for this mine
       DECB                               ; and don't start any more if it = 0
       BEQ   start_a_fireball
       JSR   activate_a_mine
       JSR   activate_a_mine
start_a_fireball:
       LDA   #$04                         ; Flag that this is a fireball
       STA   1,U
       DEC   $C8EA                        ; Decrement active bullet counter
       JMP   inc_bullet_ptr2

 ;
 ; This processes a hit on a non-fireball type mine.
 ;
not_a_fireball_mine:
       LDA   #$01
       STA   ,U
       CLR   ,Y                           ; Flag that the bullet is dead
       LDX   #score_string_pointers
       LDA   $C89B
       LDX   A,X                          ; Add mines value to players score
       LDD   14,U
       JSR   add_d_to_x_in_bcd
       LDA   4,U
       LDB   6,U
       TFR   D,X
       LDA   2,U
       LDB   #$40
       JSR   add_an_explosion
       DEC   $C8EB                        ; Decrement active mine counter
       DEC   $C8EA                        ; Decrement the active bullet counter
       JMP   inc_bullet_ptr2


 ;
 ; This routine check each active mine to see if it has
 ; collided with the command ship.
 ;
check_ship_4_mine_collision:
       LDA   $C8BD
       BNE   PEC63                        ; Do only if cmd ship not already dead
       LDA   $C8EE
       BNE   PEC63                        ; Do only if hyperspace not active
       LDY   #$C933
       LDA   #$1C
       STA   $C88F                        ; Set up a loop counter
PEC56: LDA   ,Y
       ANDA  #$3F
       BNE   check_this_active_mine
inc_mine_ptr3:
       LEAY  $12,Y
       DEC   $C88F
       BNE   PEC56
PEC63: RTS

check_this_active_mine:
       PSHS  Y
       LDA   $C8C8
       LDB   $C8CA
       TFR   D,X                          ; Load cmd ship's position
       LDA   4,Y
       LDB   6,Y
       LDY   12,Y                         ; Load mine's position
       EXG   Y,D
       JSR   check_bullet_for_hit
       PULS  Y
       BHS   inc_mine_ptr3
       CLR   ,Y                           ; Flag that the mine is dead
       CLR   $C8ED
       LDA   $C8C8
       LDB   $C8CA
       TFR   D,X
       LDA   2,Y
       ORA   #$80
       LDB   #$30
       JSR   add_an_explosion
       INC   $C8F3                        ; Flag for an explosion sound
       DEC   $C8EB                        ; Decrement active mine counter
       BRA   PEC63

 ;
 ; This routine checks to see if the command ship has collided
 ; with the enemy ship during reseeding.
 ;
check_ship_4_enemy_collision:
       LDA   $C8BD                        ; Do only if cmd ship not disabled
       BNE   PECB2
       LDA   $C8EE                        ; Don't do if hyperspace active
       BNE   PECB2
       LDA   $C8E7                        ; Check only if enemy ship visible
       BEQ   PECB2
       LDA   $C8C8
       LDB   $C8CA
       TFR   D,X                          ; Command ship location
       LDD   #$0616                       ; Size of enemy ship
       LDY   $C8DC                        ; Enemy ship's location
       JSR   check_bullet_for_hit
       BLO   cmd_and_enemy_ship_collision
PECB2: RTS

cmd_and_enemy_ship_collision:
       CLR   $C8E7                        ; Flag that enemy ship no longer visible
       CLR   $C8A2
       LDA   $C8C8
       LDB   $C8CA
       TFR   D,X
       LDA   #$08
       ORA   #$80
       LDB   #$30
       JSR   add_an_explosion
       INC   $C8F3                        ; Enable explosion sound
       RTS

 ;
 ; This routine is responsible for making miscellaneous sound,
 ; such as when an explosion occurs, or a bullet is fired.
 ;
make_misc_sounds:
PECC9: LDA   $C8F2
       BEQ   PECD6
       CLR   $C8F2
       LDU   #cmd_ship_movement_sound     ; Make cmd ship movement sound
       BRA   make_the_misc_sound
PECD6: LDA   $C8F3
       BEQ   PECE3
       CLR   $C8F3
       LDU   #explosion_sound             ; Make explosion sound
       BRA   make_the_misc_sound
PECE3: LDA   $C8B6
       BEQ   PECF0
       CLR   $C8B6
       LDU   #bullet_sound                ; Make bullet firing sound
       BRA   make_the_misc_sound
PECF0: LDA   $C8F4
       BEQ   PED00
PECF5: CLR   $C8F4
       CLR   $C8F6
       LDU   #mine_pop_sound              ; Make sound of mine 'popping' up
       BRA   make_the_misc_sound
PED00: LDA   $C8F6
       BNE   PECF5                        ; Make hyperspace sound
       BRA   PED0A

make_the_misc_sound:
       JSR   copy_bytes_2_sound_chip
PED0A: LDB   $C800
       ADDB  #$10
       CMPB  #$A0
       BHS   PED1A
       LDA   #$00
       JSR   byte_to_sound_chip
       BRA   PED20
PED1A: LDD   #$0800
       JSR   byte_to_sound_chip
PED20: LDB   $C802
       ADDB  #$20
       CMPB  #$F0
       BHS   PED30
       LDA   #$02
       JSR   byte_to_sound_chip
       BRA   PED36
PED30: LDD   #$0900
       JSR   byte_to_sound_chip
PED36: RTS


cmd_ship_movement_sound:
       DB    $00
       DB    $10
       DB    $01
       DB    $00
       DB    $06
       DB    $1F
       DB    $07
       DB    $06
       DB    $08
       DB    $0F
       DB    $FF

bullet_sound:
       DB    $02
       DB    $39
       DB    $03
       DB    $00
       DB    $06
       DB    $1F
       DB    $07
       DB    $05
       DB    $09
       DB    $0F
       DB    $FF

explosion_sound:
       DB    $06
       DB    $1F
       DB    $07
       DB    $07
       DB    $0A
       DB    $10
       DB    $0B
       DB    $00
       DB    $0C
       DB    $38
       DB    $0D
       DB    $00
       DB    $FF

mine_pop_sound:
SED5A: DB    $00
       DB    $00
       DB    $01
       DB    $00
       DB    $02
       DB    $30
       DB    $03
       DB    $00
       DB    $04
       DB    $00
       DB    $05
       DB    $00
       DB    $06
       DB    $1F
       DB    $07
       DB    $3D
       DB    $08
       DB    $00
       DB    $09
       DB    $0F
       DB    $0A
       DB    $00
       DB    $0B
       DB    $00
       DB    $0C
       DB    $00
       DB    $0D
       DB    $00
       DB    $FF

; Start of Mine Storm music block
MS_music:
       DW    MS_music_header1
       DW    music_header2_a
       DB    $00
       DB    $19
       DB    $01
       DB    $19
       DB    $00
       DB    $19
       DB    $01
       DB    $32
       DB    $00
       DB    $19
       DB    $01
       DB    $19
       DB    $00
       DB    $19
       DB    $06
       DB    $19
       DB    $05
       DB    $19
       DB    $00
       DB    $80

MS_music_header1:
       DB    $FF
       DB    $EE
       DB    $DD
       DB    $CC
       DB    $BB
       DB    $AA
       DB    $99
       DB    $88
       DB    $77
       DB    $77
       DB    $77
       DB    $77
       DB    $77
       DB    $77
       DB    $77
       DB    $77

score_string_pointers:
       DW    $C8A8                        ; Pointer to player 1 score string
       DW    $C8AF                        ; Pointer to player 2 score string

score_string_positions:
       DB    $7F                          ; Player 1 rel y score position
       DB    $A0                          ; Player 1 rel x score position
       DB    $7F                          ; Player 2 rel y score position
       DB    $10                          ; Player 2 rel x score position

mine_field_level_pointers:
       DW    $C8F9                        ; Pointer to player 1 mine field # string ;
       DW    $C900                        ; Pointer to player 2 mine field # string ;

;
 * This contains the information describing which mines
 * are to appear at the first 13 mine field levels.  The
 * values map a follows:
 *
 *  0 = dumb mine
 *  1 = magnetic mine
 *  2 = fireball mine
 *  3 = magnetic fireball mine
 *
 ;
mines_at_level_x:
       DB    $00                          ; level 1
       DB    $00
       DB    $00
       DB    $00
       DB    $02                          ; level 2
       DB    $00
       DB    $00
       DB    $00
       DB    $01                          ; level 3
       DB    $00
       DB    $00
       DB    $00
       DB    $03                          ; level 4
       DB    $00
       DB    $00
       DB    $00
       DB    $02                          ; level 5
       DB    $01
       DB    $00
       DB    $00
       DB    $02                          ; level 6
       DB    $03
       DB    $00
       DB    $00
       DB    $01                          ; level 7
       DB    $03
       DB    $00
       DB    $00
       DB    $02                          ; level 8
       DB    $02
       DB    $00
       DB    $00
       DB    $01                          ; level 9
       DB    $01
       DB    $00
       DB    $00
       DB    $03                          ; level 10
       DB    $03
       DB    $00
       DB    $00
       DB    $02                          ; level 11
       DB    $02
       DB    $02
       DB    $00
       DB    $01                          ; level 12
       DB    $01
       DB    $01
       DB    $00
       DB    $03                          ; level 13
       DB    $03
       DB    $03
       DB    $00
       DB    $80

motion_dots:
       DB    $C8
       DB    $40
       DB    $3F
       DB    $00
       DB    $20
       DB    $80
       DB    $10
       DB    $1F
       DB    $3F
       DB    $3F
       DB    $00
       DB    $BF
       DB    $BF
       DB    $BF
       DB    $C0
       DB    $20
       DB    $48
       DB    $08
       DB    $F8
       DB    $30
       DB    $A8
       DB    $10
       DB    $D0
       DB    $A0
       DB    $BF
       DB    $BF
       DB    $00
       DB    $3F
       DB    $3F
       DB    $48
       DB    $20
       DB    $80
       DB    $00
       DB    $B0
       DB    $48
       DB    $38
       DB    $FB
       DB    $38
       DB    $80
       DB    $28
       DB    $30
       DB    $48
       DB    $80
       DB    $80
       DB    $45
       DB    $F0
       DB    $28
       DB    $7F
       DB    $3F
       DB    $BF
       DB    $A5
       DB    $00
       DB    $D0
       DB    $60
       DB    $20
       DB    $28
       DB    $B8
       DB    $40
       DB    $15
       DB    $80
       DB    $40
       DB    $F8
       DB    $40
       DB    $18

mine_field:
       DB    $FA                         ; height = -6
       DB    $38                         ; width = 56
       DB    $E0                         ; rel y = -32
       DB    $C8                         ; rel x =
       DB    "MINE FIELD", $80

game_over:
       DB    $FA                         ; height = -6
       DB    $38                         ; width = 56
       DB    $E0                         ; rel y = -32
       DB    $D8                         ; rel x =
       DB    "GAME OVER",$80

dumb_mine:
       DB    $00
       DB    $10
       DB    $00
       DB    $FF
       DB    $20
       DB    $A0
       DB    $FF
       DB    $C0
       DB    $40
       DB    $FF
       DB    $90
       DB    $20
       DB    $FF
       DB    $70
       DB    $20
       DB    $FF
       DB    $50
       DB    $50
       DB    $FF
       DB    $D0
       DB    $90
       DB    $01

magnetic_mine:
       DB    $00
       DB    $20
       DB    $00
       DB    $FF
       DB    $30
       DB    $B0
       DB    $FF
       DB    $B0
       DB    $30
       DB    $FF
       DB    $B0
       DB    $D0
       DB    $FF
       DB    $30
       DB    $50
       DB    $FF
       DB    $D0
       DB    $50
       DB    $FF
       DB    $50
       DB    $D0
       DB    $FF
       DB    $50
       DB    $30
       DB    $FF
       DB    $D0
       DB    $B0
       DB    $01

fireball_mine:
       DB    $FF
       DB    $00
       DB    $00
       DB    $00
       DB    $30
       DB    $00
       DB    $FF
       DB    $10
       DB    $C0
       DB    $FF
       DB    $C0
       DB    $10
       DB    $FF
       DB    $C0
       DB    $F0
       DB    $FF
       DB    $10
       DB    $40
       DB    $FF
       DB    $F0
       DB    $40
       DB    $FF
       DB    $40
       DB    $F0
       DB    $FF
       DB    $40
       DB    $10
       DB    $FF
       DB    $F0
       DB    $C0
       DB    $01

magnetic_fireball_mine:
       DB    $FF
       DB    $00
       DB    $00
       DB    $00
       DB    $F0
       DB    $D0
       DB    $FF
       DB    $C0
       DB    $40
       DB    $FF
       DB    $20
       DB    $00
       DB    $FF
       DB    $40
       DB    $40
       DB    $FF
       DB    $00
       DB    $E0
       DB    $FF
       DB    $40
       DB    $C0
       DB    $FF
       DB    $E0
       DB    $00
       DB    $FF
       DB    $C0
       DB    $C0
       DB    $FF
       DB    $00
       DB    $20
       DB    $01

fireball:
       DB    $00
       DB    $3F
       DB    $00
       DB    $FF
       DB    $80
       DB    $00
       DB    $00
       DB    $3F
       DB    $3F
       DB    $FF
       DB    $00
       DB    $80
       DB    $01

explosion:
       DB    $FF
       DB    $7F
       DB    $20
       DB    $00
       DB    $C0
       DB    $10
       DB    $FF
       DB    $C0
       DB    $D0
       DB    $FF
       DB    $20
       DB    $7F
       DB    $00
       DB    $E0
       DB    $C0
       DB    $FF
       DB    $00
       DB    $C0
       DB    $FF
       DB    $E0
       DB    $30
       DB    $00
       DB    $C0
       DB    $00
       DB    $FF
       DB    $60
       DB    $CD
       DB    $FF
       DB    $A0
       DB    $00
       DB    $00
       DB    $20
       DB    $D0
       DB    $FF
       DB    $3C
       DB    $30
       DB    $FF
       DB    $00
       DB    $82
       DB    $00
       DB    $30
       DB    $30
       DB    $FF
       DB    $D0
       DB    $50
       DB    $FF
       DB    $20
       DB    $F0
       DB    $01

command_ship:
       DB    $00
       DB    $3F
       DB    $00
       DB    $FF
       DB    $C4
       DB    $08
       DB    $FF
       DB    $D8
       DB    $D8
       DB    $FF
       DB    $20
       DB    $00
       DB    $00
       DB    $00
       DB    $40
       DB    $FF
       DB    $E0
       DB    $00
       DB    $FF
       DB    $28
       DB    $D8
       DB    $FF
       DB    $3C
       DB    $08
       DB    $01

cmd_ship_pt1:
       DB    $00
       DB    $3F
       DB    $00
       DB    $FF
       DB    $C4
       DB    $08
       DB    $01

cmd_ship_pt2:
       DB    $00
       DB    $04
       DB    $08
       DB    $FF
       DB    $D8
       DB    $D8
       DB    $FF
       DB    $20
       DB    $00
       DB    $01

cmd_ship_pt3:
       DB    $00
       DB    $3F
       DB    $00
       DB    $FF
       DB    $C4
       DB    $F8
       DB    $01

cmd_ship_pt4:
       DB    $00
       DB    $04
       DB    $F8
       DB    $FF
       DB    $D8
       DB    $28
       DB    $FF
       DB    $20
       DB    $00
       DB    $01

enemy_ship_pt1:
       DB    $00
       DB    $20
       DB    $00
       DB    $FF
       DB    $00
       DB    $D8
       DB    $FF
       DB    $D0
       DB    $A8
       DB    $FF
       DB    $F0
       DB    $40
       DB    $FF
       DB    $08
       DB    $18
       DB    $FF
       DB    $18
       DB    $F0
       DB    $FF
       DB    $F0
       DB    $B8
       DB    $00
       DB    $10
       DB    $48
       DB    $FF
       DB    $08
       DB    $00
       DB    $FF
       DB    $E8
       DB    $10
       DB    $FF
       DB    $F8
       DB    $00
       DB    $00
       DB    $08
       DB    $00
       DB    $FF
       DB    $00
       DB    $06
       DB    $00
       DB    $10
       DB    $FA
       DB    $FF
       DB    $08
       DB    $00
       DB    $FF
       DB    $00
       DB    $F0
       DB    $00
       DB    $10
       DB    $18
       DB    $FF
       DB    $F0
       DB    $08
       DB    $01

enemy_ship_pt2:
       DB    $00
       DB    $20
       DB    $00
       DB    $FF
       DB    $00
       DB    $28
       DB    $FF
       DB    $D0
       DB    $58
       DB    $FF
       DB    $F0
       DB    $C0
       DB    $FF
       DB    $08
       DB    $E8
       DB    $FF
       DB    $18
       DB    $10
       DB    $FF
       DB    $F0
       DB    $48
       DB    $00
       DB    $10
       DB    $B8
       DB    $FF
       DB    $08
       DB    $00
       DB    $FF
       DB    $E8
       DB    $F0
       DB    $FF
       DB    $F8
       DB    $00
       DB    $FF
       DB    $08
       DB    $00
       DB    $FF
       DB    $00
       DB    $FA
       DB    $00
       DB    $10
       DB    $06
       DB    $FF
       DB    $08
       DB    $00
       DB    $FF
       DB    $00
       DB    $10
       DB    $00
       DB    $10
       DB    $E8
       DB    $FF
       DB    $F0
       DB    $F8
       DB    $01

enemy_ship_pt3:
       DB    $FF
       DB    $00
       DB    $D8
       DB    $FF
       DB    $E8
       DB    $08
       DB    $FF
       DB    $00
       DB    $40
       DB    $FF
       DB    $18
       DB    $08
       DB    $FF
       DB    $00
       DB    $D8
       DB    $00
       DB    $08
       DB    $E0
       DB    $FF
       DB    $10
       DB    $00
       DB    $FF
       DB    $00
       DB    $40
       DB    $FF
       DB    $F0
       DB    $00
       DB    $FF
       DB    $00
       DB    $C0
       DB    $01

simple_enemy_ship:
       DB    $00
       DB    $18
       DB    $00
       DB    $FF
       DB    $00
       DB    $20
       DB    $FF
       DB    $C8
       DB    $70
       DB    $FF
       DB    $10
       DB    $A0
       DB    $FF
       DB    $00
       DB    $A0
       DB    $FF
       DB    $EC
       DB    $A4
       DB    $FF
       DB    $39
       DB    $6D
       DB    $FF
       DB    $00
       DB    $20
       DB    $01

; The following is spare space in the ROM ;
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
       DB    $00
